using System;
using System.Diagnostics;
using System.IO;
using System.Linq;
using FitConnect;
using FluentAssertions;
using Newtonsoft.Json;
using NUnit.Framework;

namespace E2ETests.Subscriber;

[Order(200)]
[TestFixture]
public class SubscriberTestHappyPath : SubscriberTestBase {
    [Test]
    public void GetAvailableSubmissions_WithDestinationId_ShouldReturnSubmissionsForPickupDto() {
        // Act
        var submissions = base.subscriber.GetAvailableSubmissions(desitnationId, 0, 100).ToList();

        // Assert
        submissions.Count().Should().BeGreaterThan(0);
        submissions.All(s => s.DestinationId == desitnationId).Should().BeTrue();
    }

    [Test]
    public void GetAvailableSubmissions_WithOutDestinationId_ShouldReturnSubmissionsForPickupDto() {
        // Act
        var submissions = base.subscriber.GetAvailableSubmissions(null, 0, 100).ToList();

        // Assert
        submissions.Count().Should().BeGreaterThan(0);
    }


    [Test]
    public void GetAllSubmission_WithSubmissionId_ShouldReturnSubmissionsForPickupDto() {
        // Arrange
        int errorCounter = 0;
        var submissions = base.subscriber.GetAvailableSubmissions(null, 0, 100).ToList();
        submissions.Count().Should().BeGreaterThan(0);

        foreach (var submissionId in submissions.Select(s => s.SubmissionId)) {
            // var submissionId = "7c7bec6f-84c4-4ab5-9d3e-246b7881c8c3"; // submissions.Last().SubmissionId;

            // Act
            try {
                Console.WriteLine($"Getting submission {submissionId}");
                var dto = subscriber.RequestSubmission(submissionId);

                // Assert
                dto.Submission.Id.Should().Be(submissionId);
            }
            catch (Exception ex) {
                Console.WriteLine($"Exception getting submission {submissionId}");
                errorCounter++;
            }

            errorCounter.Should().BeLessThan(submissions.Count());
            Console.WriteLine($"Error counter: {errorCounter}/{submissions.Count()}");
            /*
             Console.WriteLine(dto.Submission.Data);
             Console.WriteLine(JsonConvert.SerializeObject(dto.Submission.Metadata));
             */
        }
    }

    [TestCase("303c60d3-3f6f-4913-bb51-4e66fe133c7f")]
    public void GetAttachment_WithSubmissionId_ShouldReturnSubmissionsForPickupDto(
        string submissionId) {
        // Act
        var attachments = subscriber.RequestSubmission(submissionId).GetAttachments();
        foreach (var attachment in attachments) {
            attachment.Content.Length.Should().BeGreaterThan(0);
            File.WriteAllBytes("attachments/test.pdf", attachment.Content);
        }
    }

    [Test]
    public void GetAttachment_FromGivenSubmission_ShouldReturnAttachment() {
        // Arrange
        int errorCounter = 0;
        var submissions = base.subscriber.GetAvailableSubmissions(null, 0, 100).ToList();
        submissions.Count().Should().BeGreaterThan(0);

        foreach (var submission in submissions) {
            Console.WriteLine(
                $"Getting submission {submission.SubmissionId} - case {submission.CaseId}");
            var submissionId = submission.SubmissionId!;
            if (!Directory.Exists($"./attachments/{submissionId}/")) {
                Directory.CreateDirectory($"./attachments/{submissionId}/");
            }

            try {
                var attachments = subscriber
                    .RequestSubmission(submissionId)
                    .GetAttachments();
                foreach (var attachment in attachments) {
                    if ((attachment?.Content?.Length ?? 0) > 0)
                        File.WriteAllBytes(
                            Path.Combine($"./attachments/{submissionId}/",
                                attachment.Filename),
                            attachment.Content);
                }

                Console.WriteLine($"Success {submissionId}");
            }
            catch (Exception ex) {
                errorCounter++;
                Directory.Delete($"./attachments/{submissionId}/");
                // Console.WriteLine($"Exception getting attachment for submission {submission.SubmissionId} - {ex.Message}");
            }
        }

        Console.WriteLine($"Error counter: {errorCounter}/{submissions.Count()}");

        Process.Start("open", "./attachments");
    }

    [TestCase("b0bd846f-f56a-4d06-a586-14fb0ea94fd0")]
    public void GetSingleSubmission_WithSubmissionId_ShouldReturnSubmissionsForPickupDto(
        string submissionId) {
        // Arrange
        var submissions = base.subscriber.GetAvailableSubmissions(null, 0, 100).ToList();
        submissions.Count().Should().BeGreaterThan(0);

        Console.WriteLine($"Getting submission {submissionId}");
        var dto = subscriber.RequestSubmission(submissionId);

        // Assert
        dto.Submission.Id.Should().Be(submissionId);


        Console.WriteLine(dto.Submission.Data);
        Console.WriteLine(JsonConvert.SerializeObject(dto.Submission.Metadata));
    }

    [TestCase("f8c95be6-6965-4ba9-9c65-78d790138c04", "cdba4941-3beb-4fe7-a610-775d81f435b0")]
    public void ConfirmSubmission_SingleSubmission(string caseId, string submissionId) {
        // Arrange
        // Act
        subscriber.AcceptSubmission(caseId, submissionId, desitnationId);
        // Assert
    }
}
