using System.IdentityModel.Tokens.Jwt;
using IdentityModel;
using Jose;
using Microsoft.Extensions.Logging;

namespace FitConnect.Encryption;

public record KeySet(string? PrivateKeyDecryption,
    string? PrivateKeySigning, string? PublicKeyEncryption, string? PublicKeySignatureVerification
);

public class FitEncryption {
    private readonly IEncryptor _encryptor = new JoseEncryptor();
    private readonly ILogger? _logger;

    public FitEncryption(ILogger? logger) {
        _logger = logger;
    }

    public FitEncryption(string? privateKeyDecryption, string? privateKeySigning,
        string? publicKeyEncryption, string? publicKeySignatureVerification, ILogger? logger) :
        this(logger) {
        PrivateKeyDecryption = privateKeyDecryption;
        PrivateKeySigning = privateKeySigning;
        PublicKeyEncryption = publicKeyEncryption;
        PublicKeySignatureVerification = publicKeySignatureVerification;
    }

    public FitEncryption(
        KeySet keySet,
        ILogger? logger) : this(logger) {
        PrivateKeyDecryption = keySet.PrivateKeyDecryption;
        PrivateKeySigning = keySet.PrivateKeySigning;
        PublicKeyEncryption = keySet.PublicKeyEncryption;
        PublicKeySignatureVerification = keySet.PublicKeySignatureVerification;
    }

    public string? PrivateKeyDecryption { get; set; }
    public string? PrivateKeySigning { get; set; }
    public string? PublicKeyEncryption { get; set; }
    public string? PublicKeySignatureVerification { get; set; }

    public (string plainText, byte[] plainBytes, byte[] tag)
        Decrypt(string cypherText, string key) {
        return _encryptor.Decrypt(key, cypherText);
    }

    public (string plainText, byte[] plainBytes, byte[] tag) Decrypt(string cypherText) {
        if (PrivateKeyDecryption == null)
            throw new InvalidOperationException("PrivateKey is not provided");

        return Decrypt(cypherText, PrivateKeyDecryption);
    }


    public string Encrypt(string plainText, string key) {
        return _encryptor.Encrypt(key, plainText);
    }

    public string Encrypt(string plain) {
        if (PublicKeyEncryption == null)
            throw new InvalidOperationException("PrivateKey is not provided");

        return Encrypt(plain, PublicKeyEncryption);
    }

    public string Encrypt(byte[] plain, string key) {
        return _encryptor.Encrypt(key, plain);
    }

    public string Encrypt(byte[] plain) {
        if (PublicKeyEncryption == null)
            throw new InvalidOperationException("PrivateKey is not provided");

        return Encrypt(plain, PublicKeyEncryption);
    }

    public JwtSecurityToken CreateSecurityEventToken(
        string submissionId,
        string caseId,
        string destinationId,
        string eventName) {
        var signingKey = Jwk.FromJson(PrivateKeySigning, new JsonMapper());
        var transactionId = "case:" + caseId;
        var subject = "submission:" + submissionId;

        var token = new JwtSecurityToken(new JwtHeader() {
            { "typ", "secevent+jwt" },
            { "kid", signingKey.KeyId },
            { "alg", "PS512" }
        }, new JwtPayload() {
            { "sub", subject },
            { "jti", Guid.NewGuid().ToString() },
            { "iat", DateTime.UtcNow.ToEpochTime() },
            { "exp", DateTime.UtcNow.AddDays(1).ToEpochTime() },
            { "iss", destinationId },
            { "events", eventName },
            { "txn", transactionId }
        });

        return token;
    }
}
