using System.Net;
using System.Net.Http.Headers;
using System.Text;
using System.Text.Json;
using Microsoft.Extensions.Logging;

namespace FitConnect.RestService;

public interface IRestCallService {
    public IWebProxy? Proxy { get; set; }
}

public abstract class RestCallService {
    private readonly string _baseUrl;
    private readonly ILogger? _logger;

    protected RestCallService(string baseUrl, ILogger? logger = null) {
        _baseUrl = baseUrl;
        _logger = logger;
    }

    public string? Token { get; set; }
    public IWebProxy? Proxy { get; set; }

    protected HttpClient CreateClient() {
        var clientHandler = new HttpClientHandler {
            Proxy = Proxy
        };
        var client = new HttpClient(clientHandler);
        client.DefaultRequestHeaders.Add("Accept", "application/json");
        if (Token != null)
            client.DefaultRequestHeaders.Add("Authorization", $"Bearer {Token}");
        return client;
    }

    protected async Task<string> RestCallForString(string endpoint, HttpMethod method,
        string? body = null, string contentType = "application/json",
        string accept = "application/json") {
        var client = CreateClient();

        var request = new HttpRequestMessage();

        if (accept != "application/json") {
            request.Headers.Accept.Clear();
            request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue(accept));
        }

        request.Method = method;
        if (body != null)
            request.Content = new StringContent(body, Encoding.UTF8, contentType);

        request.RequestUri = new Uri($"{_baseUrl}{endpoint}");
        var response = await client.SendAsync(request);

        if (response.IsSuccessStatusCode) {
            var content = await response.Content.ReadAsStringAsync();
            return content;
        }

        _logger?.LogError("Error calling {Method} - {Endpoint} with  and {Body} = {Status}", method,
            endpoint,
            body, response.StatusCode);
        throw new HttpRequestException("Error calling FitConnect API",
            new HttpRequestException(
                $"{method} -  {request.RequestUri.ToString()} - {response.StatusCode}\n" +
                await response.Content.ReadAsStringAsync()));
    }

    protected async Task<T?> RestCall<T>(string endpoint, HttpMethod method, string? body = null,
        string contentType = "application/json", string accept = "application/json")
        where T : class {
        var content = await RestCallForString(endpoint, method, body, contentType, accept);
        try {
            var result = JsonSerializer.Deserialize<T>(content);

            if (result != null)
                return result;
        }
        catch (JsonException e) {
            _logger?.LogError("Error deserializing content from {URL}\r\n{Content}",
                $"{_baseUrl}{endpoint}",
                content);
            throw new HttpRequestException("Error deserializing FitConnect API response",
                e);
        }

        throw new ArgumentException("Can not deserialize response to type " + typeof(T).Name);
    }
}
